Class: Dit
- Inherits:
-
Object
- Object
- Dit
- Defined in:
- lib/dit.rb
Overview
This is the class where all the dit work is done. The thor class is basically a very thin layer on top of this that just calls its methods directly. This is because the hooks are not running through the Thor object, but also referencing these methods.
Class Method Summary collapse
- .detect_hook(hook) ⇒ Object
- .exit_if_windows ⇒ Object
- .hook ⇒ Object
- .init ⇒ Object
- .make_dit ⇒ Object
- .make_ruby_enforcer ⇒ Object
- .prompt_for_symlink_all ⇒ Object
- .repo ⇒ Object
- .symlink(a, b) ⇒ Object
- .symlink_all ⇒ Object
- .symlink_list(list) ⇒ Object
- .symlink_unlinked ⇒ Object
- .version ⇒ Object
- .write_hook(hook_file, do_append) ⇒ Object
Class Method Details
.detect_hook(hook) ⇒ Object
90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 |
# File 'lib/dit.rb', line 90 def self.detect_hook(hook) return [false, false] unless File.exist?(hook) cannot_hook, append_to_hook = false if `cat #{hook}`.include?('./.git/hooks/dit') puts 'Dit hook already installed.' cannot_hook = true elsif `cat #{hook}`.include?('#!/usr/bin/env bash') puts "You have #{hook} hooks already that use bash, so we'll " + 'append ourselves to the file.' append_to_hook = true else puts "You have #{hook} hooks that use some foreign language, " + "so we won't interfere, but we can't hook in there." cannot_hook = true end [append_to_hook, cannot_hook] end |
.exit_if_windows ⇒ Object
28 29 30 31 32 33 34 |
# File 'lib/dit.rb', line 28 def self.exit_if_windows if OS.windows? puts 'This is a windows system, and dit does not support windows.' puts 'See vulpino/dit issue #1 if you have a potential solution.' exit 1 end end |
.hook ⇒ Object
44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 |
# File 'lib/dit.rb', line 44 def self.hook Dir.chdir(File.join('.git', 'hooks')) do # The following check for the existence of post-commit or post-merge hooks # and will not interfere with them if they exist and do not use bash. append_to_post_commit, cannot_post_commit = detect_hook 'post-commit' append_to_post_merge, cannot_post_merge = detect_hook 'post-merge' write_hook('post-commit', append_to_post_commit) unless cannot_post_commit write_hook('post-merge', append_to_post_merge) unless cannot_post_merge make_dit make_ruby_enforcer # Make sure they're executable FileUtils.chmod '+x', %w(post-commit post-merge dit force-ruby) end end |
.init ⇒ Object
13 14 15 16 17 18 19 20 21 22 23 24 25 26 |
# File 'lib/dit.rb', line 13 def self.init exit_if_windows if Dir.exist?('.git') symlink_all if prompt_for_symlink_all else Git.init(Dir.getwd) puts "Initialized empty Git repository in #{File.join(Dir.getwd, '.git')}" end hook puts 'Dit was successfully hooked into .git/hooks.' end |
.make_dit ⇒ Object
118 119 120 121 122 123 124 |
# File 'lib/dit.rb', line 118 def self.make_dit File.open('dit', 'a') do |f| f.write "#!/usr/bin/env ./.git/hooks/force-ruby\n" f.write "require 'dit'\n" f.write "Dit.symlink_unlinked\n" end end |
.make_ruby_enforcer ⇒ Object
126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 |
# File 'lib/dit.rb', line 126 def self.make_ruby_enforcer # The following lines are because git hooks do this weird thing # where they prepend /usr/bin to the path and a bunch of other stuff # meaning git hooks will use /usr/bin/ruby instead of any ruby # from rbenv or rvm or chruby, so we make a script forcing the hook # to use our ruby ruby_path = `which ruby` if ruby_path != '/usr/bin/ruby' ruby_folder = File.dirname(ruby_path) File.open('force-ruby', 'a') do |f| f.write "#!/usr/bin/env bash\n" f.write "set -e\n" if ENV['RBENV_ROOT'] # Use Rbenv's shims instead of directly going to ruby bin # By the way, if anyone has particular PATHs I should use for # RVM or chruby, please let me know! f.write "PATH=#{File.join(ENV['RBENV_ROOT'], 'shims')}:$PATH\n" else f.write "PATH=#{ruby_folder}:$PATH\n" end f.write "exec ruby \"$@\"\n" end else File.open('force-ruby', 'a') do |f| f.write "#!/usr/bin/env bash\n" f.write "exec ruby \"$@\"\n" end end end |
.prompt_for_symlink_all ⇒ Object
36 37 38 39 40 41 42 |
# File 'lib/dit.rb', line 36 def self.prompt_for_symlink_all puts 'Dit has detected an existing git repo, and will initialize it to ' + 'populate your ~ directory with symlinks.' puts 'Please confirm this by typing y, or anything else to cancel.' response = STDIN.gets.chomp.upcase response == 'Y' end |
.repo ⇒ Object
80 81 82 |
# File 'lib/dit.rb', line 80 def self.repo Git.open(Dir.getwd) end |
.symlink(a, b) ⇒ Object
84 85 86 87 88 |
# File 'lib/dit.rb', line 84 def self.symlink(a, b) File.symlink(a, b) rescue puts "Failed to symlink #{a} to #{b}" end |
.symlink_all ⇒ Object
75 76 77 78 |
# File 'lib/dit.rb', line 75 def self.symlink_all current_branch = `git rev-parse --abbrev-ref HEAD`.chomp symlink_list `git ls-tree -r #{current_branch} --name-only`.split("\n") end |
.symlink_list(list) ⇒ Object
62 63 64 65 66 67 68 69 |
# File 'lib/dit.rb', line 62 def self.symlink_list(list) list.each do |f| f.strip! wd_f = File.absolute_path f home_f = File.absolute_path(f).gsub(Dir.getwd, Dir.home) symlink wd_f, home_f end end |
.symlink_unlinked ⇒ Object
71 72 73 |
# File 'lib/dit.rb', line 71 def self.symlink_unlinked symlink_list `git show --pretty="format:" --name-only HEAD`.split("\n") end |
.version ⇒ Object
156 157 158 |
# File 'lib/dit.rb', line 156 def self.version '0.2.3' end |
.write_hook(hook_file, do_append) ⇒ Object
111 112 113 114 115 116 |
# File 'lib/dit.rb', line 111 def self.write_hook(hook_file, do_append) File.open(hook_file, 'a') do |f| f.write "#!/usr/bin/env bash\n" unless do_append f.write "( exec ./.git/hooks/dit )\n" end end |